/*
 * Written in 2019 by Magnus Bergman <magnus@snisurset.net>
 *
 * To the extent possible under law, the author(s) have dedicated all
 * copyright and related and neighboring rights to this software to the
 * public domain worldwide.
 *
 * This software is distributed without any warranty.
 *
 * See: http://creativecommons.org/publicdomain/zero/1.0/
 */
#include <abydos.h>
#include <cairo.h>
#include <cairo-svg.h>
#include <stdio.h>                      /* printf() */
#include <string.h>                     /* strrchr() */
#include <sys/stat.h>                   /* mkdir() */

#include "debug.h"

#define OUTPUT "output"

static const struct{
    const char *filename;
    const char *mime_type;
}image[] = {
#include "samples.h"
    {NULL}
};


static int
convert(const char *filename,const char *mime_type)
{
    char dest[256];
    abydos_t *ar = abydos_create(mime_type);
    const char *basename = strrchr(filename,'/');
    int res = -1;
    basename = basename ? basename+1 : filename;
    printf("> Filename     : %s\n"
           "  MIME type    : %s\n",
           basename,mime_type);
    if(!ar){
        printf("! Found no plugin capable of handling %s\n",mime_type);
        return -1;
    }
    if(abydos_from_file(ar,filename)>=0){
        abydos_info_t info;
        abydos_get_info(ar,&info);
        printf("  Dimensions   : %dx%d\n"
               "  Scalable size: %s\n",
               info.width,info.height,info.features&ABYDOS_FEATURE_SCALABLE_SIZE?"yes":"no");
        printf("  Pages        : %d\n",abydos_get_page_count(ar));
        printf("  Layers       : %d\n",abydos_get_layer_count(ar));
        printf("  Variants     : %d\n",abydos_get_variant_count(ar));
        printf("  Frames       : %d\n"
               "  Scalable time: %s\n",
               abydos_get_frame_count(ar),info.features&ABYDOS_FEATURE_SCALABLE_TIME?"yes":"no");
        if(info.features&ABYDOS_FEATURE_SCALABLE_SIZE){
            sprintf(dest,OUTPUT "/%s.svg",basename);
            //surface = cairo_svg_surface_create(dest,info.width,info.height);
            abydos_write_to_svg(ar,dest);
        }else if(abydos_get_frame_count(ar)>1){
            sprintf(dest,OUTPUT "/%s.webp",basename);
            abydos_write_to_webp(ar,dest);
        }else{
            sprintf(dest,OUTPUT "/%s.png",basename);
            abydos_write_to_png(ar,dest);
        }
        res = 0;
    }else{
        printf("! Failed to load %s: %s\n",basename,abydos_error(ar));
    }
    abydos_destroy(ar);
    return res;
}


int
main(int argc,char** argv)
{
    int i;
    int total = 0;
    int successful = 0;
    mkdir(OUTPUT,0777);
    for(i=0;image[i].filename;++i){
        if(argc>1 && strcasecmp(image[i].mime_type,argv[1]))
            continue;
        if(convert(image[i].filename,image[i].mime_type)>=0)
            ++successful;
        ++total;
    }
    printf("%d of %d files were converted successfully\n",successful,total);
    return 0;
}
